
// 모델 뷰어를 생성
var viewer = new ModelViewer();

// 모델 뷰어는 컴포넌트이다.
viewer.resize((1024 - 768) / 2, 0, 768, 768);

// 로드런너 출력
viewer.scene = new UnitModel("legs1_rdrn", "body1_sz", "arm1_sz");

var cam = viewer.camera; // 카메라

cam.target.set(0, 0.5, 0, 1); // 카메라 바라보는 곳 변경
viewer.scene.matrix.rotateY(Math.PI/5);
var camRotY = Math.PI / 4; // 카메라 회전
var zoom = 1; // 확대율
var originalDistance = 5; // 기본 카메라 거리

// 휠 감지
viewer.onmbuttonup = function () {
    var dx = mouse.mstartX - mouse.x;
    var dy = mouse.mstartY - mouse.y;
    if (dx * dx + dy * dy < 10)
    {
        zoom = 1;
        viewer.onwheel(0, 0, 0);
        cam.target.x = 0;
        cam.target.y = 0.5;
    }
};
viewer.onwheel = function(x, y, delta)
{
    // 카메라 줌
    zoom *= Math.pow(1.001, delta);
    updateCamera();
}

// 장면에 뷰어를 넣는다.
scene.attach(viewer);

var abs = Math.abs;

// 파일 드래그 드롭
scene.ondrop = function (files) {
    var file = files[0];
    
    // 파일 확장자 gx 확인
    var extidx = file.lastIndexOf('.');
    if (file.substr(extidx + 1).toLowerCase() != "gx")
    {
        message("[이게 뭐지?]"+files);
        return true;
    }

    // 유닛 부품이면 조립, 아닐경우 그냥 띄운다.
    var filename = file.substr(0, extidx);
    var model = new GxModel(filename);
    switch(model.type)
    {
    case GxModel.MP:
    case GxModel.BP:
    case GxModel.AP:
        if (!(viewer.scene instanceof UnitModel))
            viewer.scene = new UnitModel;
        viewer.scene.setPart(filename);
        break;
    default:
        viewer.scene = model;
        break;
    }

    // 루트 행렬 확인
    var mat = new Matrix;
    model.getRootMatrix(mat);
    var dist = 0;
    for (var i = 0; i < 16; i++)
    {
        var t = mat[i] - (i % 5 == 0 ? 1 : 0);
        dist += t * t;
    }
    if (dist >= 0.001) {
        message("[경고]루트 행렬이 항등 행렬이 아닙니다.");
    }
    return true;
};
scene.onkeydown = function (key) {
    // 0을 누를 경우 유닛 출력
    if (key == 0x30) {

        // 조립 변경
        // viewer.scene.assemble(1001,2001,3001);
    }

    // 1을 누를 경우 모델 출력
    if (key == 0x31) {
        // viewer.scene = new GxModel(1001);
    }

    // F5 누를 경우 스크립트 갱신
    if (key == 116)
        reload();
}

var rotateY = 0;
var rotateX = 0.005;

scene.ondraw = function () {
    ctx.__novaFont.fill("Zoom: "+ zoom.toFixed(3), 10, 100);

    if (mouse.mbutton && !mouse.mpress) {
        cam.target.x += mouse.deltaX / -200 / zoom;
        cam.target.y += mouse.deltaY / 200 / zoom;
    }
    if ((mouse.lbutton && !mouse.lpress) ||
        (mouse.rbutton && !mouse.rpress)) {
        rotateX = mouse.deltaX / 300;
        rotateY = mouse.deltaY / 300;
    }
    camRotY += rotateY;
    var lim = Math.PI / 2 - 0.01;
    if (camRotY > lim)
        camRotY = lim;
    else if (camRotY < -lim)
        camRotY = -lim;

    rotateY *= 0.75;
    viewer.scene.matrix.rotateY(rotateX);
    updateCamera();
};

function updateCamera() {
    var ndist = originalDistance / zoom;
    var campos = cam.position;
    campos.x = 0;
    campos.z = Math.cos(camRotY) * ndist;
    campos.y = Math.sin(camRotY) * ndist;
    campos.w = 0;
    campos.add(cam.target);
    cam.far = ndist * 2 + 100;
    cam.near = ndist / 100;
}